/* *************************************************************************************************************************** */
/* *************************************************************************************************************************** */
/* Project title: Earnings Management During Antidumping Investigations In Europe: Sample-Wide And Cross-Sectional Evidence  ***/
/* Date      : November, 2016                                                        *******************************************/
/* Author    : D. Godsell, M. Welker and N. Zhang                                                *******************************/
/* ********************************************************************************* *******************************************/
/* ********************************************************************************* *******************************************/

libname index 'D:\Data\datastream\indices\';
libname more "e:\more\";
libname orbis 'E:\Data\orbis\SAS\';
libname ready 'E:\Data\orbis\ready\';
libname sweden 'E:\Data\orbis\SW\';
libname master 'E:\Data\trade\master\';
libname CD 'E:\Data\orbis\CD\';
libname excel 'E:\Data\orbis\excel\';
libname capiq 'E:\data\orbis\capiq\';
libname cow "D:\Data\bilateral\";
libname parts 'E:\Data\orbis\SAS\parts\';
libname fmtlib 'D:\Data\format\';
libname status 'E:\Data\orbis\status\';
filename dummies 'D:\Data\library\dummies.sas';
%include dummies;
filename winsor 'D:\Data\library\winsor.sas';
%include winsor;
filename demean 'D:\Data\library\demean.sas';
%include demean;
filename clus2D 'D:\Data\library\clus2D.sas';
%include clus2D;
filename Famamac 'D:\Data\library\Famamac.sas';
%include Famamac;
filename FF48 'D:\Data\library\FF48.sas';
%include FF48;
filename FF10 'D:\Data\library\FF10.sas';
%include FF10;
filename FF30 'D:\Data\library\FF30.sas';
%include FF30;
filename interact 'D:\Data\library\interact.sas';
%include interact;
options VALIDVARNAME=any linesize=255 nocenter;


/* Read the 1443 individual Web-extracted Orbis data files */
/* Rename variables into Compustat convention */
%macro rename(dsetin=, dsetout=, num=);
	data &dsetout;
	set &dsetin (drop=Last_avail__year Financial_P_L_mil_USD_Last_avail Financial_expenses_mil_USD_Last_ C1 Operating_revenue__Turnover__th_ market_price market_price0 market_price1 market_price2 market_price3 market_price4 market_price5 market_price6 Closing_date_2013
	Closing_date_2012 Closing_date_2011 Closing_date_2010 Closing_date_2009 Closing_date_2008 Closing_date_2007 Closing_date_2006 Closing_date_2005 Closing_date_2004);
	where BvD_ID_number~="Credit needed";
	filenumber=&num;
	rename BvD_ID_number=bvdid;
	rename BvD_Indep__Indic_=BvD_Ind;
	rename Company_name=comname;
	rename Country_ISO_Code=ISO;

	rename Intangible_fixed_assets_mil_USD_=IFA2012;
	rename Intangible_fixed_assets_mil_USD0=IFA2011;
	rename Intangible_fixed_assets_mil_USD1=IFA2010;
	rename Intangible_fixed_assets_mil_USD2=IFA2009;
	rename Intangible_fixed_assets_mil_USD3=IFA2008;
	rename Intangible_fixed_assets_mil_USD4=IFA2007;
	rename Intangible_fixed_assets_mil_USD5=IFA2006;
	rename Intangible_fixed_assets_mil_USD6=IFA2005;
	rename Intangible_fixed_assets_mil_USD7=IFA2004;
	rename Intangible_fixed_assets_mil_USD8=IFA2003;

	rename Tangible_fixed_assets_mil_USD_20=TFA2012;
	rename Tangible_fixed_assets_mil_USD_21=TFA2011;
	rename Tangible_fixed_assets_mil_USD_22=TFA2010;
	rename Tangible_fixed_assets_mil_USD_23=TFA2009;
	rename Tangible_fixed_assets_mil_USD_24=TFA2008;
	rename Tangible_fixed_assets_mil_USD_25=TFA2007;
	rename Tangible_fixed_assets_mil_USD_26=TFA2006;
	rename Tangible_fixed_assets_mil_USD_27=TFA2005;
	rename Tangible_fixed_assets_mil_USD_28=TFA2004;
	rename Tangible_fixed_assets_mil_USD_29=TFA2003;

	rename Other_current_assets_mil_USD_201=OCA2012;
	rename Other_current_assets_mil_USD_200=OCA2011;
	rename Other_current_assets_mil_USD_202=OCA2010;
	rename Other_current_assets_mil_USD_203=OCA2009;
	rename Other_current_assets_mil_USD_204=OCA2008;
	rename Other_current_assets_mil_USD_205=OCA2007;
	rename Other_current_assets_mil_USD_206=OCA2006;
	rename Other_current_assets_mil_USD_207=OCA2005;
	rename Other_current_assets_mil_USD_208=OCA2004;
	rename Other_current_assets_mil_USD_209=OCA2003;

	rename Cash___cash_equivalent_mil_USD_2=CCEQ2012;
	rename Cash___cash_equivalent_mil_USD_0=CCEQ2011;
	rename Cash___cash_equivalent_mil_USD_1=CCEQ2010;
	rename Cash___cash_equivalent_mil_USD_3=CCEQ2009;
	rename Cash___cash_equivalent_mil_USD_4=CCEQ2008;
	rename Cash___cash_equivalent_mil_USD_5=CCEQ2007;
	rename Cash___cash_equivalent_mil_USD_6=CCEQ2006;
	rename Cash___cash_equivalent_mil_USD_7=CCEQ2005;
	rename Cash___cash_equivalent_mil_USD_8=CCEQ2004;
	rename Cash___cash_equivalent_mil_USD_9=CCEQ2003;

	rename Other_shareholders_funds_mil_USD=OSF2012;
	rename Other_shareholders_funds_mil_US0=OSF2011;
	rename Other_shareholders_funds_mil_US1=OSF2010;
	rename Other_shareholders_funds_mil_US2=OSF2009;
	rename Other_shareholders_funds_mil_US3=OSF2008;
	rename Other_shareholders_funds_mil_US4=OSF2007;
	rename Other_shareholders_funds_mil_US5=OSF2006;
	rename Other_shareholders_funds_mil_US6=OSF2005;
	rename Other_shareholders_funds_mil_US7=OSF2004;
	rename Other_shareholders_funds_mil_US8=OSF2003;

	rename Non_current_liabilities_mil_USD_=NCL2012;
	rename Non_current_liabilities_mil_USD0=NCL2011;
	rename Non_current_liabilities_mil_USD1=NCL2010;
	rename Non_current_liabilities_mil_USD2=NCL2009;
	rename Non_current_liabilities_mil_USD3=NCL2008;
	rename Non_current_liabilities_mil_USD4=NCL2007;
	rename Non_current_liabilities_mil_USD5=NCL2006;
	rename Non_current_liabilities_mil_USD6=NCL2005;
	rename Non_current_liabilities_mil_USD7=NCL2004;
	rename Non_current_liabilities_mil_USD8=NCL2003;

	rename Other_non_current_liabilities_mi=oNCL2012;
	rename Other_non_current_liabilities_m0=oNCL2011;
	rename Other_non_current_liabilities_m1=oNCL2010;
	rename Other_non_current_liabilities_m2=oNCL2009;
	rename Other_non_current_liabilities_m3=oNCL2008;
	rename Other_non_current_liabilities_m4=oNCL2007;
	rename Other_non_current_liabilities_m5=oNCL2006;
	rename Other_non_current_liabilities_m6=oNCL2005;
	rename Other_non_current_liabilities_m7=oNCL2004;
	rename Other_non_current_liabilities_m8=oNCL2003;

	rename Other_current_liabilities_mil_US=oCL2012;
	rename Other_current_liabilities_mil_U0=oCL2011;
	rename Other_current_liabilities_mil_U1=oCL2010;
	rename Other_current_liabilities_mil_U2=oCL2009;
	rename Other_current_liabilities_mil_U3=oCL2008;
	rename Other_current_liabilities_mil_U4=oCL2007;
	rename Other_current_liabilities_mil_U5=oCL2006;
	rename Other_current_liabilities_mil_U6=oCL2005;
	rename Other_current_liabilities_mil_U7=oCL2004;
	rename Other_current_liabilities_mil_U8=oCL2003;

	rename Total_shareh__funds___liab__mil_=TSFL2012;
	rename Total_shareh__funds___liab__mil0=TSFL2011;
	rename Total_shareh__funds___liab__mil1=TSFL2010;
	rename Total_shareh__funds___liab__mil2=TSFL2009;
	rename Total_shareh__funds___liab__mil3=TSFL2008;
	rename Total_shareh__funds___liab__mil4=TSFL2007;
	rename Total_shareh__funds___liab__mil5=TSFL2006;
	rename Total_shareh__funds___liab__mil6=TSFL2005;
	rename Total_shareh__funds___liab__mil7=TSFL2004;
	rename Total_shareh__funds___liab__mil8=TSFL2003;

	rename Depreciation___Amortization_mil_=DEP2012;
	rename Depreciation___Amortization_mil0=DEP2011;
	rename Depreciation___Amortization_mil1=DEP2010;
	rename Depreciation___Amortization_mil2=DEP2009;
	rename Depreciation___Amortization_mil3=DEP2008;
	rename Depreciation___Amortization_mil4=DEP2007;
	rename Depreciation___Amortization_mil5=DEP2006;
	rename Depreciation___Amortization_mil6=DEP2005;
	rename Depreciation___Amortization_mil7=DEP2004;
	rename Depreciation___Amortization_mil8=DEP2003;

	rename Research___Development_expenses_=RD2012;
	rename Research___Development_expenses0=RD2011;
	rename Research___Development_expenses1=RD2010;
	rename Research___Development_expenses2=RD2009;
	rename Research___Development_expenses3=RD2008;
	rename Research___Development_expenses4=RD2007;
	rename Research___Development_expenses5=RD2006;
	rename Research___Development_expenses6=RD2005;
	rename Research___Development_expenses7=RD2004;
	rename Research___Development_expenses8=RD2003;

	rename Operating_revenue__Turnover__mil=OPREV2012;
	rename Operating_revenue__Turnover__mi0=OPREV2011;
	rename Operating_revenue__Turnover__mi1=OPREV2010;
	rename Operating_revenue__Turnover__mi2=OPREV2009;
	rename Operating_revenue__Turnover__mi3=OPREV2008;
	rename Operating_revenue__Turnover__mi4=OPREV2007;
	rename Operating_revenue__Turnover__mi5=OPREV2006;
	rename Operating_revenue__Turnover__mi6=OPREV2005;
	rename Operating_revenue__Turnover__mi7=OPREV2004;
	rename Operating_revenue__Turnover__mi8=OPREV2003;

	rename Other_operating_expenses_mil_USD=OOE2012;
	rename Other_operating_expenses_mil_US0=OOE2011;
	rename Other_operating_expenses_mil_US1=OOE2010;
	rename Other_operating_expenses_mil_US2=OOE2009;
	rename Other_operating_expenses_mil_US3=OOE2008;
	rename Other_operating_expenses_mil_US4=OOE2007;
	rename Other_operating_expenses_mil_US5=OOE2006;
	rename Other_operating_expenses_mil_US6=OOE2005;
	rename Other_operating_expenses_mil_US7=OOE2004;
	rename Other_operating_expenses_mil_US8=OOE2003;

	rename Operating_P_L___EBIT__mil_USD_20=OPL2012;
	rename Operating_P_L___EBIT__mil_USD_21=OPL2011;
	rename Operating_P_L___EBIT__mil_USD_22=OPL2010;
	rename Operating_P_L___EBIT__mil_USD_23=OPL2009;
	rename Operating_P_L___EBIT__mil_USD_24=OPL2008;
	rename Operating_P_L___EBIT__mil_USD_25=OPL2007;
	rename Operating_P_L___EBIT__mil_USD_26=OPL2006;
	rename Operating_P_L___EBIT__mil_USD_27=OPL2005;
	rename Operating_P_L___EBIT__mil_USD_28=OPL2004;
	rename Operating_P_L___EBIT__mil_USD_29=OPL2003;

	rename P_L_for_period___Net_income__mil=PLNI2012;
	rename P_L_for_period___Net_income__mi0=PLNI2011;
	rename P_L_for_period___Net_income__mi1=PLNI2010;
	rename P_L_for_period___Net_income__mi2=PLNI2009;
	rename P_L_for_period___Net_income__mi3=PLNI2008;
	rename P_L_for_period___Net_income__mi4=PLNI2007;
	rename P_L_for_period___Net_income__mi5=PLNI2006;
	rename P_L_for_period___Net_income__mi6=PLNI2005;
	rename P_L_for_period___Net_income__mi7=PLNI2004;
	rename P_L_for_period___Net_income__mi8=PLNI2003;

	rename Extr__and_other_revenue_mil_USD_=EXREV2012;
	rename Extr__and_other_revenue_mil_USD0=EXREV2011;
	rename Extr__and_other_revenue_mil_USD1=EXREV2010;
	rename Extr__and_other_revenue_mil_USD2=EXREV2009;
	rename Extr__and_other_revenue_mil_USD3=EXREV2008;
	rename Extr__and_other_revenue_mil_USD4=EXREV2007;
	rename Extr__and_other_revenue_mil_USD5=EXREV2006;
	rename Extr__and_other_revenue_mil_USD6=EXREV2005;
	rename Extr__and_other_revenue_mil_USD7=EXREV2004;
	rename Extr__and_other_revenue_mil_USD8=EXREV2003;

	rename Extr__and_other_expenses_mil_USD=EXEXP2012;
	rename Extr__and_other_expenses_mil_US0=EXEXP2011;
	rename Extr__and_other_expenses_mil_US1=EXEXP2010;
	rename Extr__and_other_expenses_mil_US2=EXEXP2009;
	rename Extr__and_other_expenses_mil_US3=EXEXP2008;
	rename Extr__and_other_expenses_mil_US4=EXEXP2007;
	rename Extr__and_other_expenses_mil_US5=EXEXP2006;
	rename Extr__and_other_expenses_mil_US6=EXEXP2005;
	rename Extr__and_other_expenses_mil_US7=EXEXP2004;
	rename Extr__and_other_expenses_mil_US8=EXEXP2003;
	run; quit;
%mend;

proc datasets lib=work;
delete base datebase;
run; quit;

/* Assemble data files */
%macro append1 (beginnum=, endnum=);
	%do i=&beginnum %to &endnum;
		%if &i<=9 %then %do;
		%rename(dsetin=parts.file000&i, dsetout=file000&i, num=&i);
		proc append data=file000&i base=base force;
		run;

		proc datasets lib=work;
		delete file000&i;
		run; quit;
		DM 'clear log';
		%end;
		%else %if &i>=10 and &i<=99 %then %do;
		%rename(dsetin=parts.file00&i, dsetout=file00&i, num=&i);
		proc append data=file00&i base=base force;
		run;

		proc datasets lib=work;
		delete file00&i;
		run; quit;
		DM 'clear log';
		%end;
		%else %if &i>=100 and &i<=999 %then %do;
		%rename(dsetin=parts.file0&i, dsetout=file0&i, num=&i);
		proc append data=file0&i base=base force;
		run;

		proc datasets lib=work;
		delete file0&i;
		run; quit;
		DM 'clear log';
		%end;
		%else %do;
		%rename(dsetin=parts.file&i, dsetout=file&i, num=&i);
		proc append data=file&i base=base force;
		run;
		
		proc datasets lib=work;
		delete file&i;
		run; quit;
		DM 'clear log';
		%end;
	%end;
%mend;
%append1 (beginnum=1, endnum=1443);

/* Clean the datadate file */
/*
proc datasets lib=work;
delete base datebase;
run; quit;

%macro appenddate (num=);
%do i=1 %to &num;
		proc append data=fdate.fdate&i (rename=(BvD_ID_number=bvdid)) base=datebase force;
		run;
%end;
%mend;
%appenddate (num=36);

data orbis.datebase;
set datebase;
run;
*/

proc sort data=base out=clean dupout=error nodupkey;
by bvdid;
run;

proc sql;
create table pretranspose as
select a.*, b.Closing_date_2012, b.Closing_date_2011, b.Closing_date_2010, b.Closing_date_2009, b.Closing_date_2008, b.Closing_date_2007, b.Closing_date_2006, b.Closing_date_2005, b.Closing_date_2004
from clean as a, orbis.datebase as b
where a.bvdid=b.bvdid;
quit;

/* Write to the disk */
/*
data orbis.pretranspose;
set pretranspose;
run;
*/

%macro transpose(dsetin=, var=, varout=);
	data &var;
	set &dsetin;
	keep bvdid &var.2004-&var.2012;
	proc sort;
	by bvdid;
	run;

	proc transpose data=&var out=&var prefix=year;
	by bvdid;
	var &var.2004-&var.2012;
	run;
	quit;

	%if &varout=datadate %then %do;
		data &var;
		set &var (rename=(year1=&var));
		fyear=substr(_name_, %length(&var)+1, 4)+0;
		drop _name_;
		run;

		proc sql;
		create table all (where=(missing(&varout)=0)) as
		select a.bvdid, b.comname, b.iso, a.fyear, a.&var as &varout format=date9., b.sic3
		from &var as a, orbisfirm as b
		where a.bvdid=b.bvdid;
		quit;

	%end;
	%else %if &varout=AO or &varout=standard %then %do;
		data &varout;
		set &var (rename=(year1=&varout));
		fyear=substr(_name_, %length(&var)+1, 4)+0;
		drop _name_;
		run;

		proc sql;
		create table all as 
		select a.*, b.&varout
		from all as a left join &varout as b
		on a.bvdid=b.bvdid and a.fyear=b.fyear
		order by bvdid, fyear;
		quit; 
	%end;
	%else %do;
		data &varout;
		set &var (rename=(year1=&var));
		fyear=substr(_name_, %length(&var)+1, 4)+0;
		&var.1=&var+0;
		drop &var _name_;
		rename &var.1=&varout;
		run;

		proc sql;
		create table all as 
		select a.*, b.&varout
		from all as a left join &varout as b
		on a.bvdid=b.bvdid and a.fyear=b.fyear
		order by bvdid, fyear;
		quit; 
	%end;
		proc datasets lib=work;
		delete &var;
		run; quit; 

/* Correct some date mistakes 
	data all;
	set all;
	if 0<year(datadate)<1970 then datadate=mdy(12, 31, fyear);
	run;
*/
%mend;

%macro getall (dsetin=);
	proc sql;
	create table orbisfirm as
	select a.bvdid, a.comname, a.iso, a.US_SIC_Core_code__3_digits_ as sic3
	from &dsetin as a;
	quit;

	%transpose(dsetin=&dsetin, var=Closing_date_, varout=datadate);
	%transpose(dsetin=&dsetin, var=Total_assets_mil_USD_, varout=AT);
	%transpose(dsetin=&dsetin, var=IFA, varout=Intan);
	%transpose(dsetin=&dsetin, var=TFA, varout=PPENT);
	%transpose(dsetin=&dsetin, var=Current_assets_mil_USD_, varout=ACT);
	%transpose(dsetin=&dsetin, var=Stock_mil_USD_, varout=INVT);
	%transpose(dsetin=&dsetin, var=Debtors_mil_USD_, varout=RECT);
	%transpose(dsetin=&dsetin, var=CCEQ, varout=Che);
	%transpose(dsetin=&dsetin, var=Shareholders_funds_mil_USD_, varout=CEQ);
	%transpose(dsetin=&dsetin, var=Long_term_debt_mil_USD_, varout=DLTT);
	%transpose(dsetin=&dsetin, var=Current_liabilities_mil_USD_, varout=LCT);
	%transpose(dsetin=&dsetin, var=Loans_mil_USD_, varout=DLC);
	%transpose(dsetin=&dsetin, var=Creditors_mil_USD_, varout=AP);
	%transpose(dsetin=&dsetin, var=DEP, varout=DP);
	%transpose(dsetin=&dsetin, var=OPL, varout=EBIT);
	%transpose(dsetin=&dsetin, var=P_L_after_tax_mil_USD_, varout=IB);
	%transpose(dsetin=&dsetin, var=Costs_of_goods_sold_mil_USD_, varout=COGS);
	%transpose(dsetin=&dsetin, var=PLNI, varout=NI);
	%transpose(dsetin=&dsetin, var=Taxation_mil_USD_, varout=TXT);
	%transpose(dsetin=&dsetin, var=Sales_mil_USD_, varout=sale);
	%transpose(dsetin=&dsetin, var=Cash_flow_mil_USD_, varout=OANCF);
	%transpose(dsetin=&dsetin, var=OPREV, varout=turnover);
	%transpose(dsetin=&dsetin, var=Accounting_practice_, varout=standard);
	%transpose(dsetin=&dsetin, var=Capital_mil_USD_, varout=capital);
	%transpose(dsetin=&dsetin, var=Audit_status_, varout=AO);
%mend;
%getall (dsetin=orbis.pretranspose);

/* Import pretranspose */
data orbis.aftertransposemore;
set all;
run;

/* Clean legal form data */
%macro appendstatus (beginnum=, endnum=);
	%do i=&beginnum %to &endnum;
		%if &i<=9 %then %do;
		data status0&i;
		set status.lf0&i (drop=c1);
		if C2="Company name" then delete;
		rename c2=Comname;
		rename c3=Bvdid;
		rename c4=status;
		download=&i;
		if find(c4, "public", "i")>0 then public=1; else public=0;
		run;

		proc append data=status0&i (keep=bvdid status public download) base=form force;
		run;

		proc datasets lib=work;
		delete status0&i;
		run; quit; 
		%end; %else %do;

		data status&i;
		set status.lf&i (drop=c1);
		if C2="Company name" then delete;
		rename c2=Comname;
		rename c3=Bvdid;
		rename c4=status;
		download=&i;
		if find(c4, "public", "i")>0 then public=1; else public=0;
		run;

		proc append data=status&i (keep=bvdid status public download) base=form force;
		run;

		proc datasets lib=work;
		delete status&i;
		run; quit; 
		%end;
	%end;
%mend;
%appendstatus (beginnum=1, endnum=11);

/* 1. Classify obs into years 1988 to 2005. Then, export to excel. */
/*
%macro appendcd;
%do i=1 %to 16;
	proc append data=CD.a0&i base=rawcda force;
	run;
%end;
%mend appendcd;
%appendcd;

%macro selectyears;
%do i=1988 %to 2005;
	data cd.a&i;
	set rawcda (where=(Last_avail__year=&i));
	run;
%end;
%mend selectyears;
%selectyears;
*/

/* Perform manual data cleaning. */

/* 2. Transpose main variables. */
%macro transpose(dsetin=, var=, varout=, begyear=, endyear=);
	data &var;
	set &dsetin (rename=(BvD_ID_number=bvdid));
	keep bvdid &var.&begyear-&var.&endyear;
	proc sort;
	by bvdid;
	run;

	proc transpose data=&var out=&var prefix=year;
	by bvdid;
	var &var.&begyear-&var.&endyear;
	run;
	quit;

	%if &varout=datadate %then %do;
		data &var;
		set &var (rename=(year1=&var));
		fyear=substr(_name_, %length(&var)+1, 4)+0;
		&varout=mdy(substr(&var, 1, index(&var, '/') - 1), substr(&var, index(&var, '/')+1, 2), substr(&var, index(&var, '/')+4, 4));
		drop _name_;
		run;

		proc sql;
		create table c&dsetin as
		select a.bvdid, b.comname, b.iso, a.fyear, b.sic3, a.&varout format=MMDDYY10.
		from &var as a, orbisfirm as b
		where a.bvdid=b.bvdid;
		quit;
	%end;
	%else %do;
		data &varout;
		set &var (rename=(year1=&var));
		fyear=substr(_name_, %length(&var)+1, 4)+0;
		&var.1=&var+0;
		drop &var _name_;
		rename &var.1=&varout;
		run;

		proc sql;
		create table c&dsetin as 
		select a.*, (b.&varout/1000) as &varout
		from c&dsetin as a left join &varout as b
		on a.bvdid=b.bvdid and a.fyear=b.fyear
		order by bvdid, fyear;
		quit; 
	%end;
		proc datasets lib=work;
		delete &var;
		run; quit; 
%mend;

%macro getmain(dsetin=, begyear=, endyear=);
	proc sql;
	create table orbisfirm as
	select a.BvD_ID_number as bvdid, a.Company_name as comname, a.iso, a.sic3
	from &dsetin as a;
	quit;

	%transpose(dsetin=&dsetin, var=Closing_date, varout=datadate, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=AT, varout=AT, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=IFA, varout=Intan, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=TFA, varout=PPENT, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=ACT, varout=ACT, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=INVT, varout=INVT, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=RECT, varout=RECT, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=CHE, varout=CHE, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=CEQ, varout=CEQ, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=DLTT, varout=DLTT, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=LCT, varout=LCT, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=DLC, varout=DLC, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=AP, varout=AP, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=DP, varout=DP, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=EBIT, varout=EBIT, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=IB, varout=IB, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=COGS, varout=COGS, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=NI, varout=NI, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=TXT, varout=TXT, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=SALE, varout=SALE, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=OANCF, varout=OANCF, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=PRCCF, varout=PRCCF, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=CSHO, varout=CSHO, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=EE, varout=EE, begyear=&begyear, endyear=&endyear);
	%transpose(dsetin=&dsetin, var=ACO, varout=ACO, begyear=&begyear, endyear=&endyear);
%mend;

%macro transposemore(dsetin=, var=, varout=, begyear=, endyear=);
	data &var;
	set &dsetin (rename=(BvD_ID_number=bvdid));
	keep bvdid &var.&begyear-&var.&endyear;
	proc sort;
	by bvdid;
	run;

	proc transpose data=&var out=&var prefix=year;
	by bvdid;
	var &var.&begyear-&var.&endyear;
	run;
	quit;

	%if &varout=AO %then %do;
		data &var;
		set &var (rename=(year1=&var));
		fyear=substr(_name_, %length(&var)+1, 4)+0;
		&varout=&var;
		drop _name_;
		run;

		proc sql;
		create table c&dsetin as
		select a.bvdid, b.comname, a.fyear, a.&varout
		from &var as a, orbisfirm as b
		where a.bvdid=b.bvdid;
		quit;
	%end;
	%else %do;
		data &varout;
		set &var (rename=(year1=&var));
		fyear=substr(_name_, %length(&var)+1, 4)+0;
		&var.1=&var+0;
		drop &var _name_;
		rename &var.1=&varout;
		run;

		proc sql;
		create table c&dsetin as 
		select a.*, (b.&varout/1000) as &varout
		from c&dsetin as a left join &varout as b
		on a.bvdid=b.bvdid and a.fyear=b.fyear
		order by bvdid, fyear;
		quit; 
	%end;
	proc datasets lib=work;
	delete &var;
	run; quit; 
%mend;

%macro getmore(dsetin=, begyear=, endyear=);
	proc sql;
	create table orbisfirm as
	select a.BvD_ID_number as bvdid, a.Company_name as comname
	from &dsetin as a;
	quit;

	%transposemore(dsetin=&dsetin, var=Opinion, varout=AO, begyear=&begyear, endyear=&endyear);
	%transposemore(dsetin=&dsetin, var=Exchrate, varout=Exchrate, begyear=&begyear, endyear=&endyear);
	%transposemore(dsetin=&dsetin, var=Turnover, varout=Turnover, begyear=&begyear, endyear=&endyear);
	%transposemore(dsetin=&dsetin, var=XOPR, varout=XOPR, begyear=&begyear, endyear=&endyear);
%mend;

%macro clean;
	%do i=1988 %to 2005;
	    %let j=%eval(&i-9);
			data cd&i;
			set cd.cd&i;
			run;

			%getmain (dsetin=cd&i, begyear=&j, endyear=&i);
			proc append data=ccd&i base=cdmain force;
			run; quit;

			proc datasets lib=work;
			delete cd&i ccd&i;
			run; quit;

			data a&i;
			set cd.a&i;
			run;

			%getmore (dsetin=a&i, begyear=&j, endyear=&i);
			proc append data=ca&i base=cdmore force;
			run; quit;
			
			proc datasets lib=work;
			delete ca&i a&i status&i;
			run; quit;
		%end;
%mend;
%clean;

proc sql;
create table cdpart as
select a.*, b.AO, b.exchrate, b.turnover, b.xopr, missing(b.bvdid) as missingadd
from cdmain as a left join cdmore as b
on a.bvdid=b.bvdid and a.fyear=b.fyear
order by bvdid, fyear, turnover descending;
quit;

proc sort data=cdpart nodupkey;
by bvdid fyear;
run; quit;

data cleanCD; 
set cdpart;
if abs(fyear-year(datadate))>=2 and month(datadate)<=5 then do; fyear=year(datadate)-1; correct=1; end;
if abs(fyear-year(datadate))>=2 and month(datadate)>5  then do; fyear=year(datadate); correct=1; end;
run;

data findit;
set cleancd;
auditor=(1-missing(AO));
run;

proc sql;
create table orbis.updatedCD2 (drop=oldbvdid) as
select a.*, coalesce(b.newid, a.oldbvdid) as bvdid, a.sic3 as psic
from cleancd (rename=(bvdid=oldbvdid)) as a left join orbis.dictionary as b
on a.oldbvdid=b.oldid
order by bvdid, fyear;
quit;

data orbis.dictionary;
set dictionary;
run;

proc datasets lib=work;
delete cdstatus2;
run; quit;

%macro getCDstatus;
	%do i=1988 %to 2005;
		data status&i;
		set cd.a&i (rename=(bvd_id_number=bvdid company_name=comname));
		keep comname bvdid status Public;
		Public = (find(status, "public", "i")>0);
		run;

		proc append data=status&i base=cdstatus2 force;
		run; quit;
		
		proc datasets lib=work;
		delete status&i;
		run; quit;
	%end;
%mend;
%getCDstatus;

/* Translation */
proc sql;
create table orbis.cdstatus(drop=oldbvdid) as
select a.*, coalesce(b.newid, a.oldbvdid) as bvdid
from cdstatus2 (rename=(bvdid=oldbvdid)) as a left join orbis.dictionary as b
on a.oldbvdid=b.oldid
order by bvdid;
quit;

/* Assemble the CD-based and web-extracted data together */
proc sql;
create table first as
select bvdid, comname, iso, fyear, datadate format=date9., sic3, at, intan, ppent, act, invt, rect, che, ceq, dltt, lct, dlc, ap, dp, ebit, ib, cogs, ni, txt, coalesce(turnover, sale) as sale, oancf, standard
from orbis.aftertranspose;

create table part1 as
select bvdid, comname, iso, fyear, datadate format=date9., sic3, at, intan, ppent, act, invt, rect, che, ceq, dltt, lct, dlc, ap, dp, ebit, ib, cogs, ni, txt, coalesce(turnover, sale) as sale, . as oancf, 0 as web, "Local GAAP" as standard
from orbis.updatedCDl (where=(missing(datadate)=0));

create table part2 as
select bvdid, comname, iso, fyear, datadate format=date9., sic3, at, intan, ppent, act, invt, rect, che, ceq, dltt, lct, dlc, ap, dp, ebit, ib, cogs, ni, txt, sale, oancf, 1 as web, standard
from first;
quit;

proc append data=part1 base=part2 force;
run; quit;

/* Form the complete panel of 20 years data; rename the BvDid into GVKEY. */
proc sql;
create table rawEUcompustat (where=(EU=1 and at>0 and missing(sale)=0)) as
select a.bvdid as gvkey, a.comname, a.iso, a.fyear, a.datadate, a.at, a.intan, a.ppent, a.act, a.invt, a.rect, a.che, a.ceq, a.dltt, a.lct, a.dlc, a.ap, a.dp, a.ebit, a.ib, a.cogs, a.ni, 
a.txt, a.sale, a.oancf, input(a.sic3, best4.) as sic3, input(a.sic3, best4.)*10 as sic4, a.web, case when a.ISO in ("AT", "BE", "BG", "HR", "CY", "CZ", "DK", "EE", "FI", "FR", "DE", "GR",
, "IE", "IT", "LT", "LU", "MT", "NL", "PL", "PT", "RO", "SK", "SI", "ES", "SE", "GB") then 1 else 0 end as EU, a.standard
from part2 as a
order by bvdid, fyear, web descending, sale descending;

create table rawEUcompustat (where=(EU=1 and at>0 and missing(sale)=0)) as
select a.*, b.countryid
from rawEUcompustat as a, orbis.EUID as b
where a.iso=b.iso;
quit;

/* Clear some duplicate years */
proc sort data=rawEUcompustat nodupkey;
by gvkey fyear;
run;

/* Attach public/private status variable */
proc sql;
create table rawEUcompustat2 as 
select a.*, b.public
from rawEUcompustat as a left join orbis.webstatus as b
on a.gvkey=b.bvdid;
quit;

proc sort data=rawEUcompustat2 dupout=error out=sorted nodupkey;
by gvkey fyear;
run;

data filled blank;
set rawEUcompustat2;
if missing(public)=0 then output filled;
if missing(public)=1 then output blank;
run;

proc sql;
create table blank as 
select a.*, b.public
from blank (drop=public) as a left join orbis.cdstatus as b
on a.gvkey=b.bvdid;
quit;

proc append data=blank base=filled force;
run;

data EUcompustat;
set filled;
if missing(public)=1 then public=0;
proc sort nodupkey;
by gvkey fyear;
run;

%macro accruals (dsetin=, dsetout=);
	proc datasets lib=work;
	delete allcountry;
	run; quit;

	data a1;
	set &dsetin;
	ib=coalesce(ib, ni);
	if 60<=int(sic3/10)<=69 or 49<=int(sic3/10)<=49 then regulate=1; else regulate=0;
	run;

	DM 'clear log';
		%do i=1 %to 26;
			proc sql;
			create table a1b as
			select a.*, n(fyear) as nobs
			from a1 (where=(countryid=&i)) as a
			group by gvkey
			having calculated nobs>=3
			order by gvkey, fyear;
			quit;

			proc sort data=a1b out=a1c dupout=error nodupkey;
			by gvkey fyear;
			run;

			options nonotes error=10;
			proc expand data=a1c  out=a2  method=none;
			by gvkey;
			id fyear;
			convert datadate=lag1datadate / transform = (lag 1);
			convert at=lag1at / transform = (lag 1);
			convert at=lag2at / transform = (lag 2);
			convert at=lead1at / transform = (lead 1);
			convert at=lead2at / transform = (lead 2);
			convert rect=lag1rect / transform = (lag 1);
			convert che=lag1che / transform = (lag 1);
			convert act=lag1act / transform = (lag 1);
			convert lct=lag1lct / transform = (lag 1);
			convert dlc=lag1dlc / transform = (lag 1);
			convert dlc=lag2dlc / transform = (lag 2);
			convert dlc=lead1dlc / transform = (lead 1);
			convert dltt=lag1dltt / transform = (lag 1);
			convert dltt=lag2dltt / transform = (lag 2);
			convert dltt=lead1dltt / transform = (lead 1);
			convert ceq=lag1ceq / transform = (lag 1);
			convert ceq=lag2ceq / transform = (lag 2);
			convert ceq=lead1ceq / transform = (lead 1);
			convert ib=lag1ib / transform = (lag 1);
			convert ib=lead1ib / transform = (lead 1);
			convert ni=lead1ni / transform = (lead 1);
			convert ni=lag1ni / transform = (lag 1);
			convert fyear=lag1fyear / transform = (lag 1);
			convert sale=lag1sale / transform = (lag 1);
			convert sale=lag2sale / transform = (lag 2);
			convert sale=lag3sale / transform = (lag 3);
			run;

			data a2;
			set a2;
			chgact=act-lag1act;
			chglct=lct-lag1lct;
			chgche=che-lag1che;
			chgdlc=dlc-lag1dlc;
			wcaccrual=(chgact-chglct-chgche+chgdlc);
			totacc=(chgact-chglct-chgche+chgdlc-dp);
			swcaccrual=(chgact-chglct-chgche+chgdlc)/lag1at;
			stotacc=(chgact-chglct-chgche+chgdlc-dp)/lag1at;
			chgsale=(sale-lag1sale)/lag1at;
			sppe=ppegt/lag1at;
			chgsale_rect=(sale-lag1sale-rect+lag1rect)/lag1at;
			CFO=(IB-totacc);
			atgrowth=(at-lag1at)/lag1at;
			ssale=(sale/at);
			drop lag1fyear;
			NOA=(CEQ-CHE+DLC+DLTT)/sale; /* Net operating asset scaled by sales */
			run;

			options nonotes;
			proc expand data=a2 out=country&i method=none;
			by gvkey;
			id fyear;
			convert CFO=lag1CFO / transform = (lag 1);
			convert CFO=lead1CFO / transform = (lead 1);
			convert CFO=lag2CFO / transform = (lag 2);
			convert CFO=lead2CFO / transform = (lead 2);
			convert fyear=lag1fyear / transform = (lag 1);
			convert fyear=lead1fyear / transform = (lead 1);
			convert ssale=lag1ssale / transform = (lag 1);
			convert ssale=lag2ssale / transform = (lag 2);
			convert ssale=lag3ssale / transform = (lag 3);
			convert noa=lag1noa / transform = (lag 1);
			run;
			options notes;

			proc append data=country&i base=allcountry;
			run; quit;

			proc datasets lib=work;
			delete country&i;
			run; quit;
			DM 'clear log';
		%end;

	data &dsetout;
	set allcountry;
	slag1cfo=lag1cfo/lag1at;
	slead1cfo=lead1cfo/lag1at;
	scfo=cfo/lag1at;
	invat=1/lag1at;
	roa=ib/lag1at;
	stdsales=std(ssale, lag1ssale, lag2ssale);
	stdsales4=std(ssale, lag1ssale, lag2ssale, lag3ssale);
	%FFI10(SIC4);
	%FFI30(SIC4);
	%FFI48(SIC4);
	if missing(FFI30)=1 then FFI30=31;
	if missing(FFI48)=1 then FFI48=49;
	stdcfo=std(cfo/at, lag1cfo/lag1at, lag2cfo/lag2at);
	chgroa=roa-lag1ib/lag2at;
	run;
%mend;
%accruals (dsetin=EUcompustat, dsetout=EUcomp);

/* Do a fuzzy match here by squeezing the dates */
proc sql;
create table euprematch as
select a.*, b.P_AD_DUTY, input(substr(b.P_AD_DUTY, 1, 5), best6.) as padduty, b.F_AD_DUTY, input(substr(b.F_AD_DUTY, 1, 5), best6.) as fadduty, input(substr(b.P_AD_MARGIN, 1, 5), best6.) as padmargin
from orbis.euprematch as a left join master.master7 as b
on a.case_id=b.case_id;

create table multimatch (where=(missing(gvkey)=0)) as
select a.case_id, a.case_repcode, a.inv_cty_code, a.d_firm, a.PRODUCT, a.initdate, a.pinjdate, a.paddate, a.pdumpdate, a.finjdate, a.faddate, a.fdumpdate, a.padmargin, b.quality, 
coalesce(b.main_bvd, b.rawbvdid) as gvkey, b.country, b.matchcount, case when a.F_INJ_DEC="N" then 0 else 1 end as injury, a.padduty, a.fadduty, a.F_INJ_DEC
from euprematch as a left join orbis.EUrawmatch2 as b
on a.d_firm=b.d_firm and a.d_firm~="MI"
order by case_id;
quit;

%macro squeeze(dsetin=, dsetout=, keydate=);
	proc sql;
	create table &dsetout as
	select a.*, a.&keydate-b.lag1datadate as backdays, b.datadate-a.&keydate as forwarddays, case when (b.datadate-a.&keydate)<183 then 0 else 1 end as i, b.datadate, 
	b.lag1datadate label='lag1datadate', case when missing(a.case_repcode)=0 then 1 else 0 end as event, b.fyear as yeart, b.iso
	from &dsetin as a left join EUcomp as b
	on a.gvkey=b.gvkey and coalesce(b.lag1datadate, b.datadate-366)<=a.&keydate<b.datadate
	order by gvkey, fyear, &keydate;

	create table &dsetout as
	select a.*, b.fyear, int(b.sic3/100) as sic1, int(b.sic3/10) as sic2, b.sic3, b.at, b.atgrowth, b.roa, b.ffi48, FFI48_desc, b.ffi30, FFI30_desc, b.ffi10, FFI10_desc, b.lag1at, b.lag2at,
	b.comname, b.public, b.standard, b.dltt, b.lag1dltt, b.lag1dlc, b.ceq, b.lag1ceq, b.ib, b.ni, b.lead1dltt, b.lead1dlc, b.lead1ceq, b.lead1ni, b.lead1ib, b.lag1ni, b.lag1ib, b.lag2ceq,
	b.lag2dltt, b.lag2dlc, b.lag1sale, b.lag2sale, b.lag3sale, b.lag1ib/b.lag2at as lag1roa, b.lead1ni/b.at as lead1roa
	from &dsetout as a, Eucomp as b
	where a.gvkey=b.gvkey and a.yeart=b.fyear+a.i
	order by gvkey, fyear, &keydate;
	quit;
%mend squeeze;
%squeeze(dsetin=multimatch, dsetout=eventfirmsbase, keydate=initdate);

%macro numericalize(dsetin=, dsetout=, indclass=, prefix=);
	proc sort data=&dsetin out=&indclass.only nodupkey;
	by &indclass;
	run;

	data &indclass.only;
	set &indclass.only;
	indnum=_n_;
	run;

	proc sql;
	create table &dsetout as
	select a.*, b.indnum as &prefix
	from &dsetin as a, &indclass.only as b
	where a.&indclass=b.&indclass;
	quit;
%mend numericalize;

/* Import manually checked file */
data checked;
set orbis.manualrevised (where=(correct=1)) ;
run;
%numericalize(dsetin=checked, dsetout=checked, indclass=iso, prefix=isonum);
%numericalize(dsetin=checked, dsetout=checked, indclass=gvkey, prefix=gvkeynum);

proc sql;
create table eventfirms0 as 
select a.*, b.correct, b.matchcount as matchcount0
from eventfirmsbase as a, checked as b
where a.d_firm=b.d_firm and a.gvkey=b.gvkey
order by gvkey, fyear;
quit;

proc sql;
create table checked as 
select *, count(distinct gvkeynum) as casegvkeycount
from checked as a
group by case_id;

create table checked as 
select *, count(distinct isonum) as caseisocount
from checked as a
group by case_repcode;
quit;

proc sort data=orbis.fulldistance out=adddistance nodupkey;
by case_id;
run; quit;

proc sql;
create table checked as
select a.*, log(coalesce(b.holisticdist, 1)) as logdistance, log(coalesce(b.holisticdist, 1))/a.casegvkeycount as logavgdistance, coalesce(b.holisticdist, 1) as totdistance
from checked as a left join adddistance as b
on a.case_id=b.case_id;
quit;

data sortedrepcode2;
set orbis.sortedrepcode2;
exporters=exporters_pct+0;
proc sort nodupkey;
by case_repcode;
run; quit;

proc means data=sortedrepcode2 noprint;
by case_repcode;
output out=finalduty (drop=_type_ _freq_) mean(exporters)=exporters_pct;
run; quit;

proc sql;
create table eventfirms0 as 
select a.*, b.correct, b.matchcount as matchcount0, caseisocount, casegvkeycount, b.logdistance, b.logavgdistance, b.totdistance
from eventfirmsbase as a, checked as b
where a.d_firm=b.d_firm and a.gvkey=b.gvkey
order by gvkey, fyear;
quit;

proc sort data=orbis.bilateral2 out=bilateral nodupkey;
by case_id iso INV_CTY_CODE year;
run;

proc sql;
create table bilateral as 
select distinct case_id, year as fyear, INV_CTY_CODE, iso, per as ctyfrac
from bilateral
order by case_id, INV_CTY_CODE, iso, fyear;
quit;

proc sort data=eventfirms0 out=eventfirms0 nodupkey;
by gvkey fyear INV_CTY_CODE;
run; quit;

proc sql;
create table eventfirms0 as 
select a.*, sum(b.ctyfrac) as sumctyfrac
from eventfirms0 as a left join bilateral as b
on a.case_id=b.case_id and a.fyear=b.fyear and a.iso=b.iso and a.INV_CTY_CODE=b.INV_CTY_CODE
group by a.gvkey, a.fyear
order by a.gvkey, a.fyear;
quit;

proc sort data=eventfirms0 out=eventfirms1 dupout=duplicate nodupkey;
by gvkey fyear;
run; quit;

proc sql;
create table testsample0 as
select a.*, b.civ_com as civil, b.eff_jud as judicial, b.risk_exp as risk, b.rule_law, (b.eff_jud+b.rule_law) as institution
from eventfirms1 as a left join orbis.la_porta as b
on a.iso=b.iso2;

create table testsample0 as 
select a.*, coalesce(b.exporters_pct, 0) as exporters_pct, case when a.iso in ('AT', 'BE', 'DE', 'FI', 'FR', 'GB', 'SE', "NL") then 1 else 0 end as strong
from testsample0 as a left join finalduty as b
on a.case_repcode=b.case_repcode
order by strong;

create table testsample0 as
select a.*, b.regquality as governance
from testsample0 as a left join orbis.WGI as b
on a.iso=b.iso2 and a.fyear=b.year;

create table testsample0 as
select a.*, case when b.confirmpublic=1 then 1 else 0 end as confirmpublic, monotonic() as marker
from testsample0 as a left join orbis.overhaul2 as b
on a.gvkey=b.gvkey;
quit;

proc sort data=orbis.ipo out=ipo nodupkey;
by gvkey;
run; quit;

proc sql;
create table testsample0 as 
select a.*, b.ipodate, b.question, (b.legal_form="Public") as public, c.tradefree, d.gdp
from testsample0 (drop=public) as a left join ipo as b
on a.gvkey=b.gvkey left join orbis.tradefree as c
on a.iso=c.iso2 and a.fyear=c.year left join orbis.finalindexjar as d
on a.iso=d.iso2 and a.fyear=d.year;
quit;

proc means data=testsample0 n mean median std;
var governance sumctyfrac casegvkeycount logdistance logavgdistance;
run; quit;

/* Import the indices from the Brown et al. JBFA paper. Process enforcement score. */
proc sql;
create table rawenforce as
select a.*, b.enforce, c.total
from orbis.audit as a left join orbis.enforce as b
on a.iso=b.iso and a.year=b.year left join orbis.total as c
on a.iso=c.iso and a.year=c.year;

create table enforceisolist as
select iso
from orbis.audit
where year=2002;

create table enforceisolist as
select a.*, b.year
from enforceisolist as a, orbis.year as b
order by iso, year;

create table linear as
select a.*, b.audit, b.enforce, b.total, case when a.year<2002 then 2002 when a.year>2008 then 2008 end as psuedoyear
from enforceisolist as a left join rawenforce as b
on a.iso=b.iso and a.year=b.year
order by iso, year;
quit;

/* Data has some holes. Now Fill the holes */
proc expand data=linear out=linear;
by iso;
id year;
convert audit=linearaudit / method=join;
convert enforce=linearenforce / method=join;
convert total=lineartotal / method=join;
run;

proc sql;
create table enforcescore (where=(1992<=year<=2012)) as
select a.iso, a.year, coalesce(a.audit, a.linearaudit, b.audit) as auditscore, coalesce(a.enforce, a.linearenforce, b.enforce) as enforcescore, 
coalesce(a.total, a.lineartotal, b.total) as totalscore
from linear as a left join rawenforce as b
on a.iso=b.iso and a.psuedoyear=b.year
order iso, year;
quit;

proc means data=orbis.enforcescore mean median;
var auditscore enforcescore totalscore;
run; quit;

proc sql;
create table testsample0 as
select a.*, coalesce(b.auditscore, 0) as auditscore, coalesce(b.enforcescore, 0) as enforcescore, coalesce(b.totalscore, 0) as totalscore, coalesce(c.enfscore, 0) as staffscore
from testsample0 as a left join orbis.enforcescore as b
on a.iso=b.iso and a.fyear=b.year left join orbis.enfscore as c
on a.iso=c.iso2 and a.fyear=c.year;
quit;

proc sql;
create table wgi as 
select distinct iso2, year, regquality
from orbis.wgi (where=(iso2 in ("AT", "BE", "CZ", "DE", "ES", "FI", "FR", "GB", "GR", "IT", "LT", "NL", "PL", "PT", "SE", "SI", "SK") and 1992<=year<=2012))
order by iso2, year;

create table UWA as 
select distinct iso as iso2, year, auditscore, enforcescore, totalscore
from orbis.enforcescore (where=(iso in ("AT", "BE", "CZ", "DE", "ES", "FI", "FR", "GB", "GR", "IT", "LT", "NL", "PL", "PT", "SE", "SI", "SK") and 1992<=year<=2012))
order by iso, year;

create table panelregscore as 
select distinct a.*, b.auditscore, b.enforcescore, b.totalscore
from wgi as a left join uwa as b
on a.iso2=b.iso2 and a.year=b.year 
order by iso2, year;

create table laporta as
select iso2, civ_com as civil, eff_jud as judicial, risk_exp as risk, rule_law, (eff_jud+rule_law) as institution
from orbis.la_porta (where=(iso2 in ("AT", "BE", "CZ", "DE", "ES", "FI", "FR", "GB", "GR", "IT", "LT", "NL", "PL", "PT", "SE", "SI", "SK")));
quit;

proc sql;
create table testsample1 as
select a.*, coalesce(b.eufrac, 0) as eufrac, sum(coalesce(b.eufrac, 0)) as caseeufrac
from testsample0 as a left join orbis.eufraction as b
on a.inv_cty_code=b.inv_cty_code and a.fyear=b.fyear
group by case_repcode;
quit;

proc means data=testsample1 n mean median std;
var auditscore enforcescore totalscore staffscore institution governance exporters_pct sumctyfrac;
run; quit;

proc means data=testsample1 noprint;
output out=mediancalc median(sumctyfrac)=mediansumctyfrac median(exporters_pct)=medianexporters_pct median(logdistance)=medianlogdistance;
run; quit;

proc sql;
create table testsample2 as 
select a.*, b.mediansumctyfrac, b.medianexporters_pct, b.medianlogdistance
from testsample1 as a, mediancalc as b;
quit;

data testsample;
set testsample2;
chgdltt=(lead1dltt+lead1dlc-lag1dltt-lag1dlc);
SEO=lead1ceq-lag1ceq-ib-lead1ib;
high=(exporters_pct>=medianexporters_pct);
if SEO>0.05*(mean(lag1at, at)) or chgdltt>0.05*(mean(lag1at, at)) then newfin=1; else newfin=0;
if chgdltt>0.05*(mean(lag1at, at)) then newdebt=1; else newdebt=0;
if SEO>0.05*(mean(lag1at, at)) then newequity=1; else newequity=0;
if roa<0 or lag1roa<0 or lead1roa<0 then loss=1; else loss=0;
nonewfin=1-newfin;
good=(governance>=1.18);
large=(sumctyfrac>=mediansumctyfrac);
more=(casegvkeycount>2);
few=1-more;
near=(0<logdistance<=medianlogdistance);
*if high=1;   /* partition 1: injury-based */
*if exporters_pct>=0.5; /* partition 1': injury-based */
*if few=1; /* partition 2: Coordination */
*if near=1; /* partition 2': distance */
*if large=1; /* partition 3: Retaliation? */
*if newfin=0; /* partition 4: New financing? */
*if strong=1; /* partition 5: La Porta */
*if good=1;   /* partition 5': Reg-quality */

/* More on the regulatory quality partition based on the calculated medians. */
audit=(auditscore>15);
enforce=(enforcescore>9);
total=(totalscore>24);
run;

proc sort data=testsample dupout=error1 nodupkey;
by gvkey fyear;
run; quit;

/* Extend the petition list */
%macro extend(extend=, dsetin=, dsetout=, indclass=);
	%if &extend=yes %then %do;
	proc sort data=&dsetin out=&dsetout nodupkey;
	by &indclass fyear;
	run;

	proc sql;
	create table &dsetout as
	select a.firmname as suspectname, b.gvkey, b.conm as firmname, b.psic, b.&indclass, b.fyear, a.lag1at as suspectat, b.lag1at as peerat, abs(a.lag1at-b.lag1at) as compareat
	from &dsetout as a, EUcomp (where=(at>0)) as b
	where a.&indclass=b.&indclass and a.fyear=b.fyear
	order by &indclass, fyear, compareat;
	quit;

	proc rank data=&dsetout (where=(compareat>=0)) out=&dsetout (where=(rcompareat<=4));
	by &indclass fyear;
	var compareat;
	ranks rcompareat;
	run;
	%end;

	%else %do;
	data &dsetout;
	set &dsetin;
	run;
	%end;
%mend;
%extend(extend=no, dsetin=testsample, dsetout=eventfirms2, indclass=ffind);

/* Generate a list of petitioners */
proc sort data=eventfirmsbase out=eventnames (keep=gvkey) nodupkey;
by gvkey;
run;

/* Partition the Compustat universe into event and control samples: by year. Suspect firms are from eventfirms2, in which fyear is the identified event year. */
proc sql;
create table eventsample as
select a.*, b.d_firm as firmname, b.high, b.newfin
from EUcomp as a, eventfirms2 as b
where a.gvkey=b.gvkey and a.fyear=b.fyear
order by gvkey, fyear;

create table controlsample (where=(control=1 and fyear-lag1fyear=1 and lead1fyear-fyear=1)) as
select a.*, case when missing(b.gvkey)=1 then 1 else 0 end as control
from EUcomp as a left join eventnames as b
on a.gvkey=b.gvkey
order by gvkey, fyear;
quit;

%macro getvariable(dsetin=, dsetfrom=, dsetout=, event=);
	proc sql;
	create table &dsetout as
	select distinct b.*, &event as event
	from &dsetin as a, &dsetfrom as b
	where a.gvkey=b.gvkey
	order by gvkey, fyear;
	quit;
%mend;

%macro matching(peerfirm=, dsetin=, dsetout=, industry=, indclass=);
	%if &peerfirm=yes %then %do;
		%if &industry=yes %then %do;
			proc sql;
			create table matched as
			select a.gvkey, a.firmname, a.fyear, a.lag1at, b.lag1at as peerlag1at, b.gvkey as peergvkey, abs(b.lag1at-a.lag1at) as lag1atdiff
			from eventsample as a, controlsample as b
			where a.&indclass=b.&indclass and a.fyear=b.fyear
			order by gvkey, fyear, calculated lag1atdiff;
			quit;
		%end;
		%else %do;
			proc sql;
			create table matched (where=(lag1atdiff>0)) as
			select a.gvkey, a.firmname, a.fyear, a.lag1at, b.lag1at as peerlag1at, b.gvkey as peergvkey, b.comname as peername, abs(b.lag1at-a.lag1at) as lag1atdiff, abs(b.lag1at-a.lag1at)/a.lag1at as atratio
			from eventsample as a, controlsample as b
			where a.fyear=b.fyear and a.iso=b.iso and a.&indclass~=b.&indclass and a.roa-0.1<=b.roa<=a.roa+0.1 and a.public=b.public and b.regulate=0
			and missing(b.chgsale_rect)=0 and missing(b.cfo)=0
			order by gvkey, fyear, calculated lag1atdiff;
			quit;
		%end;

		proc rank data=matched out=matched (where=(rlag1atdiff<=5));
		by gvkey fyear;
		var lag1atdiff;
		ranks rlag1atdiff;
		run;

		proc sort data=matched (keep=gvkey) out=suspect nodupkey;
		by gvkey;
		run;

		proc sort data=matched (keep=peergvkey) out=peer (rename=(peergvkey=gvkey)) nodupkey;
		by peergvkey;
		run;

		/* Form a complete panel */
		%getvariable(dsetin=suspect, dsetfrom=EUcomp, dsetout=suspect, event=1);
		%getvariable(dsetin=peer, dsetfrom=controlsample, dsetout=peer, event=0);

		proc sql;
		create table suspect as
		select a.*, case when a.fyear=b.fyear then 1 else 0 end as eventyear
		from suspect as a left join eventfirms2 as b
		on a.gvkey=b.gvkey and a.fyear=b.fyear
		order by gvkey, fyear;

		create table peer as
		select a.*, case when a.fyear=b.fyear then 0 else 0 end as eventyear
		from peer as a left join matched as b
		on a.gvkey=b.peergvkey and a.fyear=b.fyear
		order by gvkey, fyear;
		quit;
		
		proc append data=suspect base=peer force;
		run;	

		data &dsetout;
		set peer;
		run;
	%end;
	%else %do;
		%getvariable(dsetin=eventsample, dsetout=&dsetout, event=1);
		proc sql;
		create table &dsetout as
		select a.*, case when a.fyear=b.fyear then 1 else 0 end as eventyear
		from &dsetout as a left join eventfirms2 as b
		on a.gvkey=b.gvkey and a.fyear=b.fyear
		order by gvkey, fyear;
		quit;
	%end;
	proc sort data=&dsetout out=&dsetout nodupkey;
	by gvkey fyear;
	run;
%mend matching;
%matching(peerfirm=yes, dsetin=eventsample, dsetout=matchcomplete, industry=no, indclass=ffi30);

%macro rollingyear(dsetin=, dsetout=);
	data &dsetout;
	set &dsetin;
	run;
	%do i=1 %to 3;
		proc sql;
		create table &dsetout as
		select a.*, case when missing(b.fyear)=0 then 1 else 0 end as lead&i.eventyear
		from &dsetout as a left join eventfirms2 as b
		on a.gvkey=b.gvkey and a.fyear=b.fyear+&i;
		quit;

		proc sql;
		create table &dsetout as
		select a.*, case when missing(b.fyear)=0 then 1 else 0 end as lag&i.eventyear
		from &dsetout as a left join eventfirms2 as b
		on a.gvkey=b.gvkey and a.fyear=b.fyear-&i;
		quit;
	%end;
%mend;
%rollingyear(dsetin=matchcomplete, dsetout=complete);

/* Create year indicators. */
%macro createdummies(dsetin=, dsetout=, indclass=, prefix=);
	proc sort data=&dsetin out=&indclass.only nodupkey;
	by &indclass;
	run;

	data &indclass.only;
	set &indclass.only;
	&prefix.num=_n_;
	run;

	proc sql;
	create table &dsetin.1temp as
	select a.*, b.&prefix.num 
	from &dsetin as a, &indclass.only as b
	where a.&indclass=b.&indclass;
	quit;
	%dummies(dset=&dsetin.1temp, var=&prefix.num, pre=&prefix);

	data &dsetout;
	set &dsetin.1temp;
	run;
%mend;
%createdummies(dsetin=complete, dsetout=complete1, indclass=iso, prefix=isocountry);
%createdummies(dsetin=complete1, dsetout=complete1, indclass=ffi30, prefix=indclass);
%dummies(dset=complete1, var=fyear, pre=year);

proc sql;
create table complete2 as
select a.*, medret, b.medexret
from complete1 as a left join capiq.indret30 as b
on a.iso=b.iso2 and a.fyear=b.fyear and a.ffi30=b.ffi30;
quit;

proc sort data=complete2 nodupkey;
by gvkey fyear;
run; 

data complete2;
set complete2 (where=(fyear>=1992));
if missing(lag3eventyear)=1 then lag3eventyear=0;
if missing(lag2eventyear)=1 then lag2eventyear=0;
if missing(lag1eventyear)=1 then lag1eventyear=0;
if missing(lead1eventyear)=1 then lead1eventyear=0;
if missing(lead2eventyear)=1 then lead2eventyear=0;
if missing(lead3eventyear)=1 then lead3eventyear=0;
/*
if lag3eventyear=1 or lag2eventyear=1 or lag1eventyear then pre=1; else pre=0;
if eventyear=1 or lead1eventyear=1 then current=1; else current=0;
if lead2eventyear=1 or lead3eventyear=1 then post=1; else post=0; 
*/
if lag3eventyear=1 or lag2eventyear=1 or lag1eventyear=1 then pre=1; else pre=0;
if eventyear=1 or lead1eventyear=1 then current=1; else current=0;
if lead2eventyear=1 or lead3eventyear=1 then post=1; else post=0; 
if lag3eventyear=1 then evtyear=-3;
if lag2eventyear=1 then evtyear=-2;
if lag1eventyear=1 then evtyear=-1;
if     eventyear=1 then evtyear=0;
if lead1eventyear=1 then evtyear=1;
if lead2eventyear=1 then evtyear=2;
if lead3eventyear=1 then evtyear=3;
if lag3eventyear+lag2eventyear+lag1eventyear+eventyear+lead1eventyear+lead2eventyear+lead3eventyear=0 then noneventyear=1;
lntotass=log(at);
lnsale=log(sale);
leverage=(dlc+dltt)/at;
salegrowth=(sale-lag1sale)/lag1sale;
ssalelag1=sale/lag1at;
scfolag3eventyear=scfo*lag3eventyear;
scfolag2eventyear=scfo*lag2eventyear;
scfolag1eventyear=scfo*lag1eventyear;
scfoeventyear=scfo*eventyear;
scfolead1eventyear=scfo*lead1eventyear;
scfolead2eventyear=scfo*lead2eventyear;
scfolead3eventyear=scfo*lead3eventyear;

if post=1 and current=1 then do; post=0; recode=1; end;
if pre=1 and current=1 then do; pre=0; recode=1; end;
if pre=1 and post=1 then do; post=0; recode=1; end;

if eventyear=1 and lag3eventyear=1 then do; lag3eventyear=0; recode2=1; end;
if eventyear=1 and lag2eventyear=1 then do; lag2eventyear=0; recode2=1; end;
if eventyear=1 and lag1eventyear=1 then do; lag1eventyear=0; recode2=1; end;
if eventyear=1 and lead1eventyear=1 then do; lead1eventyear=0; recode2=1; end;
if eventyear=1 and lead2eventyear=1 then do; lead2eventyear=0; recode2=1; end;
if eventyear=1 and lead3eventyear=1 then do; lead3eventyear=0; recode2=1; end;

if lead1eventyear=1 and lag3eventyear=1 then do; lag3eventyear=0; recode2=1; end;
if lead1eventyear=1 and lag2eventyear=1 then do; lag2eventyear=0; recode2=1; end;
if lead1eventyear=1 and lag1eventyear=1 then do; lag1eventyear=0; recode2=1; end;
if lead1eventyear=1 and lead2eventyear=1 then do; lead2eventyear=0; recode2=1; end;
if lead1eventyear=1 and lead3eventyear=1 then do; lead3eventyear=0; recode2=1; end;

if lead2eventyear=1 and lag3eventyear=1 then do; lag3eventyear=0; recode2=1; end;
if lead2eventyear=1 and lag2eventyear=1 then do; lag2eventyear=0; recode2=1; end;
if lead2eventyear=1 and lag1eventyear=1 then do; lag1eventyear=0; recode2=1; end;

if lead3eventyear=1 and lag3eventyear=1 then do; lag3eventyear=0; recode2=1; end;
if lead3eventyear=1 and lag2eventyear=1 then do; lag2eventyear=0; recode2=1; end;
if lead3eventyear=1 and lag1eventyear=1 then do; lag1eventyear=0; recode2=1; end;

if lag2eventyear=1 and lag3eventyear=1 then do; lag2eventyear=0; recode2=1; end;
if lag2eventyear=1 and lag1eventyear=1 then do; lag1eventyear=0; recode2=1; end;
if lead3eventyear=1 and lead2eventyear=1 then do; lead3eventyear=0; recode2=1; end;

chgsale_rectlag3eventyear =chgsale_rect*lag3eventyear;
chgsale_rectlag2eventyear =chgsale_rect*lag2eventyear;
chgsale_rectlag1eventyear =chgsale_rect*lag1eventyear;
chgsale_recteventyear	  =chgsale_rect*eventyear;
chgsale_rectlead1eventyear=chgsale_rect*lead1eventyear;
chgsale_rectlead2eventyear=chgsale_rect*lead2eventyear;
chgsale_rectlead3eventyear=chgsale_rect*lead3eventyear;

/* More interactions */
medexretlag3eventyear     =medexret*lag3eventyear;
medexretlag2eventyear =medexret*lag2eventyear;
medexretlag1eventyear =medexret*lag1eventyear;
medexreteventyear	  =medexret*eventyear;
medexretlead1eventyear=medexret*lead1eventyear;
medexretlead2eventyear=medexret*lead2eventyear;
medexretlead3eventyear=medexret*lead3eventyear;

chgsale_rectpre =chgsale_rect*pre;
chgsale_rectcurrent =chgsale_rect*current;
chgsale_rectpost =chgsale_rect*post;

medexretpre     =medexret*pre;
medexretcurrent=medexret*current;
medexretpost =medexret*post;

scfopre=scfo*pre;
scfocurrent=scfo*current;
scfopost=scfo*post;
slag1cfopre=slag1cfo*pre;
slag1cfocurrent=slag1cfo*current;
slag1cfopost=slag1cfo*post;
slead1cfopre=slead1cfo*pre;
slead1cfocurrent=slead1cfo*current;
slead1cfopost=slead1cfo*post;

/* More descriptive data */
schgact=(chgact-chgche)/lag1at;
schglct=(chglct-chgdlc)/lag1at;
sdpexp=dp/lag1at;
scaleap=ap/lag1at;
scalear=rect/lag1at;
run;
%winsor(dsetin=complete2, dsetout=complete2, byvar=fyear, vars=stotacc swcaccrual invat slag1cfo slead1cfo scfo sppe chgsale ssalelag1 chgsale_rect at lntotass lnsale leverage salegrowth
stdsales stdsales4 stdcfo lag1noa medret medexret scfopre scfocurrent scfopost slag1cfopre slag1cfocurrent slag1cfopost slead1cfopre slead1cfocurrent slead1cfopost
chgsale_rectlag3eventyear chgsale_rectlag2eventyear chgsale_rectlag1eventyear chgsale_recteventyear	chgsale_rectlead1eventyear chgsale_rectlead2eventyear chgsale_rectlead3eventyear
medexretlag3eventyear medexretlag2eventyear medexretlag1eventyear medexreteventyear	medexretlead1eventyear medexretlead2eventyear medexretlead3eventyear
chgsale_rectpre chgsale_rectcurrent chgsale_rectpost medexretpre medexretcurrent medexretpost schgact schglct sdpexp scaleap scalear, type=winsor, pctl=1 99);

/* Send this final file to Stata */
%macro tostata(dsetin=, dsetout=);
	proc export data=&dsetin file="D:\Stata\trade\EU\final\&dsetout" DBMS=STATA REPLACE;
	run; quit;

	data &dsetout;
	set complete2;
	run;
%mend;
%tostata(dsetin=complete2, dsetout=baseline);

/* Sample composition */
proc sort data=testsample out=uniquefirms nodupkey;
by gvkey;
run;

%macro casebreakdown(dsetin=, byvar=, addinfo=);
	proc sort data=&dsetin out=&dsetin.count;
	by &byvar;
	run; quit;

	proc means data=&dsetin.count noprint;
	by &byvar;
	&addinfo;
	output out=&byvar.count mean(at)=meanstotacc;
	run; quit;
%mend casebreakdown;
%casebreakdown(dsetin=testsample, byvar=iso, addinfo=);
%casebreakdown(dsetin=testsample, byvar=ffi10, addinfo=id FFI10_desc);
%casebreakdown(dsetin=testsample, byvar=fyear, addinfo=);

%casebreakdown(dsetin=uniquefirms, byvar=iso, addinfo=);
%casebreakdown(dsetin=uniquefirms, byvar=ffi10, addinfo=id FFI10_desc);

%macro firmbreakdown(dsetin=, byvar=, addinfo=);
	proc sort data=&dsetin (where=(event=1 and nmiss(stotacc, swcaccrual, invat, chgsale_rect, sppe, slag1cfo, scfo, slead1cfo, lnsale, leverage, salegrowth, stdsales, lag1noa, medexret)=0 and event=1)) out=eventcount;
	by &byvar;
	run; quit;

	proc means data=eventcount noprint;
	by &byvar;
	&addinfo;
	output out=&byvar.firmcount mean(at)=meanstotacc;
	run;
%mend firmbreakdown;
%firmbreakdown(dsetin=complete2, byvar=iso);
%firmbreakdown(dsetin=complete2, byvar=ffi10, addinfo=id FFI10_desc);
%firmbreakdown(dsetin=complete2, byvar=fyear);

/* Summary statistics: */
proc sort data=complete2 out=sumstat;
by evtyear;
run; quit;

%macro summarystat(condition=, tablename=);
	proc printto print="D:\stata\trade\EU\final\&tablename..txt" new;
	run;

	proc means data=sumstat n  mean  p25 median p75;
	where &condition and nmiss(stotacc, swcaccrual, invat, chgsale_rect, sppe, slag1cfo, scfo, slead1cfo, lnsale, leverage, salegrowth, stdsales, lag1noa, medexret)=0;
	title &condition;
	var stotacc swcaccrual invat sppe chgsale_rect slag1cfo scfo slead1cfo lnsale leverage salegrowth stdsales lag1noa medexret;
	run; quit;

	proc printto;
	run;
%mend summarystat;
%summarystat(condition=event=1, tablename=event);
%summarystat(condition=event=0, tablename=matched);
%summarystat(condition=event<10, tablename=full);

proc printto print='D:\stata\trade\EU\byyear';
run;

proc datasets lib=work;
delete byyearbase byyearaccruals;
run; quit;

%macro eventyearstats(dsetin=, byvar=, evntyear=);
	proc means data=&dsetin noprint;
	where missing(slead1cfo)=0 and missing(slag1cfo)=0 and event=1 and &byvar=1;
	id &byvar;
	output out=byyear(drop=_type_) mean(stotacc swcaccrual ROA scfo)=stotacc swcaccrual ROA scfo p25(stotacc swcaccrual ROA scfo)=stotacc25 swcaccrual25 ROA25 scfo25 
	median(stotacc swcaccrual ROA scfo)=dstotacc dswcaccrual dROA dscfo p75(stotacc swcaccrual ROA scfo)=stotacc75 swcaccrual75 ROA75 scfo75;
	run; quit;

	proc sql;
	create table byyear as
	select _freq_ as n, stotacc, stotacc25, dstotacc, stotacc75, 1 as delimit1, swcaccrual, swcaccrual25, dswcaccrual, swcaccrual75, 1 as delimit2, scfo, scfo25, dscfo, scfo75
	from byyear;
	quit;

	proc append data=byyear base=byyearbase force;
	run; quit;
%mend;
%eventyearstats(dsetin=sumstat, byvar=noneventyear, evntyear=-99);
%eventyearstats(dsetin=sumstat, byvar=lag3eventyear, evntyear=-3);
%eventyearstats(dsetin=sumstat, byvar=lag2eventyear, evntyear=-2);
%eventyearstats(dsetin=sumstat, byvar=lag1eventyear, evntyear=-1);
%eventyearstats(dsetin=sumstat, byvar=eventyear, evntyear=0);
%eventyearstats(dsetin=sumstat, byvar=lead1eventyear, evntyear=1);
%eventyearstats(dsetin=sumstat, byvar=lead2eventyear, evntyear=2);
%eventyearstats(dsetin=sumstat, byvar=lead3eventyear, evntyear=3);


proc datasets lib=work;
delete matrixhigh matrixfew matrixpartialnear matrixlarge matrixnonewfin matrixgood matrixstrong matrixaudit matrixenforce matrixtotal;
run; quit;

%macro creatematrix(dsetin=, column=, row=);
	proc means data=&dsetin noprint;
	where &column=1 and &row=1;
	output out=see (drop=_type_ _freq_) n(fyear)=&column;
	run; quit;
	
	data see;
	set see;
	row="&row";
	run;

	proc append data=see base=matrix&column force;
	run; quit;

	proc datasets lib=work;
	delete see;
	run; quit;
%mend;
%creatematrix(dsetin=testsample, column=high, row=high);
%creatematrix(dsetin=testsample, column=high, row=few);
%creatematrix(dsetin=testsample, column=high, row=partialnear);
%creatematrix(dsetin=testsample, column=high, row=large);
%creatematrix(dsetin=testsample, column=high, row=nonewfin);
%creatematrix(dsetin=testsample, column=high, row=good);
%creatematrix(dsetin=testsample, column=high, row=strong);
%creatematrix(dsetin=testsample, column=high, row=audit);
%creatematrix(dsetin=testsample, column=high, row=enforce);
%creatematrix(dsetin=testsample, column=high, row=total);

%creatematrix(dsetin=testsample, column=few, row=high);
%creatematrix(dsetin=testsample, column=few, row=few);
%creatematrix(dsetin=testsample, column=few, row=partialnear);
%creatematrix(dsetin=testsample, column=few, row=large);
%creatematrix(dsetin=testsample, column=few, row=nonewfin);
%creatematrix(dsetin=testsample, column=few, row=good);
%creatematrix(dsetin=testsample, column=few, row=strong);
%creatematrix(dsetin=testsample, column=few, row=audit);
%creatematrix(dsetin=testsample, column=few, row=enforce);
%creatematrix(dsetin=testsample, column=few, row=total);

%creatematrix(dsetin=testsample, column=partialnear, row=high);
%creatematrix(dsetin=testsample, column=partialnear, row=few);
%creatematrix(dsetin=testsample, column=partialnear, row=partialnear);
%creatematrix(dsetin=testsample, column=partialnear, row=large);
%creatematrix(dsetin=testsample, column=partialnear, row=nonewfin);
%creatematrix(dsetin=testsample, column=partialnear, row=good);
%creatematrix(dsetin=testsample, column=partialnear, row=strong);
%creatematrix(dsetin=testsample, column=partialnear, row=audit);
%creatematrix(dsetin=testsample, column=partialnear, row=enforce);
%creatematrix(dsetin=testsample, column=partialnear, row=total);

%creatematrix(dsetin=testsample, column=large, row=high);
%creatematrix(dsetin=testsample, column=large, row=few);
%creatematrix(dsetin=testsample, column=large, row=partialnear);
%creatematrix(dsetin=testsample, column=large, row=large);
%creatematrix(dsetin=testsample, column=large, row=nonewfin);
%creatematrix(dsetin=testsample, column=large, row=good);
%creatematrix(dsetin=testsample, column=large, row=strong);
%creatematrix(dsetin=testsample, column=large, row=audit);
%creatematrix(dsetin=testsample, column=large, row=enforce);
%creatematrix(dsetin=testsample, column=large, row=total);

%creatematrix(dsetin=testsample, column=nonewfin, row=high);
%creatematrix(dsetin=testsample, column=nonewfin, row=few);
%creatematrix(dsetin=testsample, column=nonewfin, row=partialnear);
%creatematrix(dsetin=testsample, column=nonewfin, row=large);
%creatematrix(dsetin=testsample, column=nonewfin, row=nonewfin);
%creatematrix(dsetin=testsample, column=nonewfin, row=good);
%creatematrix(dsetin=testsample, column=nonewfin, row=strong);
%creatematrix(dsetin=testsample, column=nonewfin, row=audit);
%creatematrix(dsetin=testsample, column=nonewfin, row=enforce);
%creatematrix(dsetin=testsample, column=nonewfin, row=total);

%creatematrix(dsetin=testsample, column=good, row=high);
%creatematrix(dsetin=testsample, column=good, row=few);
%creatematrix(dsetin=testsample, column=good, row=partialnear);
%creatematrix(dsetin=testsample, column=good, row=large);
%creatematrix(dsetin=testsample, column=good, row=nonewfin);
%creatematrix(dsetin=testsample, column=good, row=good);
%creatematrix(dsetin=testsample, column=good, row=strong);
%creatematrix(dsetin=testsample, column=good, row=audit);
%creatematrix(dsetin=testsample, column=good, row=enforce);
%creatematrix(dsetin=testsample, column=good, row=total);

%creatematrix(dsetin=testsample, column=strong, row=high);
%creatematrix(dsetin=testsample, column=strong, row=few);
%creatematrix(dsetin=testsample, column=strong, row=partialnear);
%creatematrix(dsetin=testsample, column=strong, row=large);
%creatematrix(dsetin=testsample, column=strong, row=nonewfin);
%creatematrix(dsetin=testsample, column=strong, row=good);
%creatematrix(dsetin=testsample, column=strong, row=strong);
%creatematrix(dsetin=testsample, column=strong, row=audit);
%creatematrix(dsetin=testsample, column=strong, row=enforce);
%creatematrix(dsetin=testsample, column=strong, row=total);

%creatematrix(dsetin=testsample, column=audit, row=high);
%creatematrix(dsetin=testsample, column=audit, row=few);
%creatematrix(dsetin=testsample, column=audit, row=partialnear);
%creatematrix(dsetin=testsample, column=audit, row=large);
%creatematrix(dsetin=testsample, column=audit, row=nonewfin);
%creatematrix(dsetin=testsample, column=audit, row=good);
%creatematrix(dsetin=testsample, column=audit, row=strong);
%creatematrix(dsetin=testsample, column=audit, row=audit);
%creatematrix(dsetin=testsample, column=audit, row=enforce);
%creatematrix(dsetin=testsample, column=audit, row=total);

%creatematrix(dsetin=testsample, column=enforce, row=high);
%creatematrix(dsetin=testsample, column=enforce, row=few);
%creatematrix(dsetin=testsample, column=enforce, row=partialnear);
%creatematrix(dsetin=testsample, column=enforce, row=large);
%creatematrix(dsetin=testsample, column=enforce, row=nonewfin);
%creatematrix(dsetin=testsample, column=enforce, row=good);
%creatematrix(dsetin=testsample, column=enforce, row=strong);
%creatematrix(dsetin=testsample, column=enforce, row=audit);
%creatematrix(dsetin=testsample, column=enforce, row=enforce);
%creatematrix(dsetin=testsample, column=enforce, row=total);

%creatematrix(dsetin=testsample, column=total, row=high);
%creatematrix(dsetin=testsample, column=total, row=few);
%creatematrix(dsetin=testsample, column=total, row=partialnear);
%creatematrix(dsetin=testsample, column=total, row=large);
%creatematrix(dsetin=testsample, column=total, row=nonewfin);
%creatematrix(dsetin=testsample, column=total, row=good);
%creatematrix(dsetin=testsample, column=total, row=strong);
%creatematrix(dsetin=testsample, column=total, row=audit);
%creatematrix(dsetin=testsample, column=total, row=enforce);
%creatematrix(dsetin=testsample, column=total, row=total);

proc sql;
create table fullmatrix as
select a.row, a.high, b.few, c.partialnear, d.large, e.nonewfin, g.strong, f.good, h.audit, i.enforce, j.total
from matrixhigh as a, matrixfew as b, matrixpartialnear as c, matrixlarge as d, matrixnonewfin as e, matrixstrong as g, matrixgood as f, matrixaudit as h, matrixenforce as i, matrixtotal as j
where a.row=b.row and a.row=c.row and a.row=d.row and a.row=e.row and a.row=f.row and a.row=g.row and a.row=h.row and a.row=i.row and a.row=j.row;
quit;

